/**  @file kdumpfile.h
 * Public interface for `libkdumpfile` (kernel coredump file access).
*/
/* Copyright (C) 2014-2016 Petr Tesarik <ptesarik@suse.com>

   This file is free software; you can redistribute it and/or modify
   it under the terms of either

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at
       your option) any later version

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at
       your option) any later version

   or both in parallel, as here.

   libkdumpfile is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _KDUMPFILE_H
#define _KDUMPFILE_H	1

#include <stddef.h>
#include <stdint.h>
#include <unistd.h>

#include "addrxlat.h"

/** Major version (1st number in the release tag). */
#define KDUMPFILE_VER_MAJOR	0
/** Minor version (2nd number in the release tag). */
#define KDUMPFILE_VER_MINOR	3
/** Micro version (3rd number in the release tag). */
#define KDUMPFILE_VER_MICRO	0

/** Make a single-number version from three digits of the release tag. */
#define KDUMPFILE_MKVER(major,minor,micro)	\
	(((major) << 16) | ((minor) << 8) | ((micro)))

/** Version as a single number */
#define KDUMPFILE_VERSION		\
	KDUMPFILE_MKVER(		\
		KDUMPFILE_VER_MAJOR,	\
		KDUMPFILE_VER_MINOR,	\
		KDUMPFILE_VER_MICRO)

/** Version as a string constant */
#define KDUMPFILE_VERSION_STRING "0.3.0"

#ifdef  __cplusplus
extern "C" {
#endif

/**  Type of a generic number.
 *
 * This type is large enough to hold register value on any architecture
 * supported by `libkdumpfile`. Note that it may be larger than the
 * registers in the target.
 */
typedef uint_fast64_t kdump_num_t;

/**  Type of a physical or virtual address.
 *
 * This type is large enough to hold any possible address type on any
 * architecture supported by `libkdumpfile`. Note that this type may
 * be larger than the actual address in the target.
 */
typedef addrxlat_addr_t kdump_addr_t;

/**  Maximum value represented by @ref kdump_addr_t.
 */
#define KDUMP_ADDR_MAX	ADDRXLAT_ADDR_MAX

/**  Type of a physical address.
 *
 * Use this type instead of @ref kdump_addr_t if the entity is always
 * known to be a physical address.
 */
typedef kdump_addr_t kdump_paddr_t;

/**  Type of a virtual addresses.
 *
 * Use this type instead of @ref kdump_addr_t if the entity is always
 * known to be a virtual address.
 */
typedef kdump_addr_t kdump_vaddr_t;

/**  Representation of a dump file.
 *
 * The context contains all information needed to work with a dump file.
 * It is an opaque type, so it can be modified and/or extended without
 * breaking binary compatibility with existing programs.
 */
typedef struct _kdump_ctx kdump_ctx_t;

/**  Status code.
 *
 * Return type of all (almost) library functions that may fail. It
 * provides a very rough description of the error. See @ref kdump_get_err
 * if you want a human-readable error description.
 */
typedef enum _kdump_status {
	KDUMP_OK = 0,		/**< Success. */
	KDUMP_ERR_SYSTEM,	/**< OS error, see @c errno. */
	KDUMP_ERR_NOTIMPL,	/**< Unimplemented feature. */
	KDUMP_ERR_NODATA,	/**< Data is not stored in the dump file. */
	KDUMP_ERR_CORRUPT,	/**< Corrupted file data. */
	KDUMP_ERR_INVALID,	/**< Invalid value. */
	KDUMP_ERR_NOKEY,	/**< No such attribute key. */
	KDUMP_ERR_EOF,		/**< Unexpected EOF. */
	KDUMP_ERR_BUSY,		/**< Too many pending requests. */
	KDUMP_ERR_ADDRXLAT,	/**< Address translation error. */
} kdump_status;

/**  Target dump byte order.
 *
 * Target byte order. Note that this may be different from the host byte
 * order. The library will convert any values it needs internally, but
 * if you read any data from the dump file, you are responsible for
 * converting the data to the host byte order.
 *
 * @sa KDUMP_ATTR_BYTE_ORDER
 */
typedef enum _kdump_byte_order {
	KDUMP_BIG_ENDIAN,
	KDUMP_LITTLE_ENDIAN,
} kdump_byte_order_t;

/**  Type of a Xen dump.
 * @sa KDUMP_ATTR_XEN_TYPE
 */
typedef enum _kdump_xen_type {
	KDUMP_XEN_NONE,		/**< Not running under Xen */
	KDUMP_XEN_SYSTEM,	/**< Comlete dump of a Xen machine */
	KDUMP_XEN_DOMAIN,	/**< Dump of a single domain */
} kdump_xen_type_t;

/**  Xen physmap translation type.
 * @sa KDUMP_ATTR_XEN_XLAT
 */
typedef enum _kdump_xen_xlat {
	KDUMP_XEN_AUTO,		/**< Auto-translated physmap */
	KDUMP_XEN_NONAUTO	/**< Non-auto-translated physmap */
} kdump_xen_xlat_t;

/**  Initialize a new dump file object.
 * @returns    New initialized object, or @c NULL on failure.
 *
 * Use this function to create a new @ref kdump_ctx_t. When the object
 * is no longer needed, you should free all resources with
 * @ref kdump_free.
 */
kdump_ctx_t *kdump_new(void);

/**  Clone a dump file object.
 * @param orig  Original dump file object.
 * @returns     Cloned dump file object, or @c NULL on allocation error.
 *
 * A cloned dump file object shares file descriptor, cache, attributes and
 * other data with the original. This data remains shared, i.e. changes
 * in one of the objects are also seen by the other object.
 *
 * The following fields are *not* shared:
 * - address translation context (see @ref kdump_get_addrxlat_ctx),
 * - error string (see @ref kdump_get_err),
 *
 * Private data and callbacks are initialized to their values from the
 * @p orig context here.
 *
 * @sa @ref threads
 */
kdump_ctx_t *kdump_clone(const kdump_ctx_t *orig);

/**  Free a dump file object.
 * @param ctx  Object to be freed.
 *
 * Free all resources associated with the dump file. Do not just call
 * @c free(ctx), because that may leak some resources.
 *
 * The object must not be used after calling this function.
 */
void kdump_free(kdump_ctx_t *ctx);

/** Prepend an error message.
 * @param ctx     Dump file object.
 * @param status  Error status.
 * @param msgfmt  Message format string (@c printf style).
 * @returns       The error status which was passed as argument.
 *
 * This function prepends the new error message to the existing
 * content of the error buffer, resulting in a kind of error
 * backtrace.
 */
kdump_status kdump_err(
	kdump_ctx_t *ctx, kdump_status status, const char *msgfmt, ...)
	__attribute__ ((format (printf, 3, 4)));

/** Clear the error message.
 * @param ctx  Dump file object.
 */
void kdump_clear_err(kdump_ctx_t *ctx);

/**  Get a detailed error string.
 * @param ctx  Dump file object.
 * @returns    Error string, or @c NULL if there was no error.
 *
 * If an error status is returned, this function can be used to get
 * a human-readable description of the error. The error string is not
 * reset by calling this function, but it is reset by calling any
 * library function that returns @ref kdump_status.
 */
const char *kdump_get_err(kdump_ctx_t *ctx);

/**  Get the associated address translation context.
 * @param ctx  Dump file object.
 * @returns    Associated address translation context.
 *
 * This function grabs a new reference to the address translation context.
 * You should call addrxlat_ctx_decref on the returned object when you
 * no longer need it.
 */
addrxlat_ctx_t *kdump_get_addrxlat_ctx(const kdump_ctx_t *ctx);

/**  Get the associated address translation system.
 * @param ctx  Dump file object.
 * @returns    Associated address translation system.
 *
 * This function grabs a new reference to the address translation system.
 * You should call addrxlat_sys_decref on the returned object when you
 * no longer need it.
 */
addrxlat_sys_t *kdump_get_addrxlat_sys(const kdump_ctx_t *ctx);

/**  Address spaces used by kdump_read()
 *
 * When passing an address to kdump_read(), this type is used to
 * specify the kind of address.
 *
 * The difference between @c KDUMP_KPHYSADDR and @c KDUMP_MACHPHYSADDR
 * matters only in environments where the kernel has a different view
 * of physical address space than the CPU, e.g. paravirtualized kernels
 * under Xen.
 */
typedef enum _kdump_addrspace {
	KDUMP_KPHYSADDR,	/**< Kernel physical address. */
	KDUMP_MACHPHYSADDR,	/**< Machine physical address. */
	KDUMP_KVADDR,		/**< Kernel virtual address. */
} kdump_addrspace_t;

/**  Read data from the dump file.
 * @param ctx              Dump file object.
 * @param[in] as           Address space of @c addr.
 * @param[in] addr         Any type of address.
 * @param[out] buffer      Buffer to receive data.
 * @param[in,out] plength  Length of the buffer.
 * @returns                Error status.
 *
 * Read data from a dump file. On return, the length of the buffer pointed
 * to by @c plength is updated to reflect the actual number of bytes read
 * from the dump file.
 *
 * If @c kdump_read returns @ref KDUMP_OK, then all requested bytes have
 * been read successfully, and the value referenced by @p plength is not
 * changed. However, if the read fails, some bytes may already have been
 * read, and their number is reported in this variable, so the caller may
 * process a partial read.
 *
 * In particular, if some pages were filtered out, @c kdump_read may return
 * @ref KDUMP_ERR_NODATA when it reaches the filtered out page.
 */
kdump_status kdump_read(kdump_ctx_t *ctx,
			 kdump_addrspace_t as, kdump_addr_t addr,
			 void *buffer, size_t *plength);

/**  Read a string from the dump file.
 * @param ctx        Dump file object.
 * @param[in] as     Address space of @c addr.
 * @param[in] addr   Any type of address.
 * @param[out] pstr  String to be read.
 * @returns          Error status.
 *
 * Use this function to read a NUL-terminated string at address @c addr.
 * The resulting string is allocated dynamically, and you should free
 * it with the @c free library function when it is no longer needed.
 * This function is usually more efficient than implementing the same
 * thing with @ref kdump_read.
 */
kdump_status kdump_read_string(kdump_ctx_t *ctx,
			       kdump_addrspace_t as, kdump_addr_t addr,
			       char **pstr);

/**  Dump file attribute value type.
 */
typedef enum _kdump_attr_type {
	KDUMP_NIL,		/**< No type; used for deletions. */
	KDUMP_DIRECTORY,	/**< Attribute directory. */
	KDUMP_NUMBER,		/**< General number. */
	KDUMP_ADDRESS,		/**< Address or symbol value. */
	KDUMP_STRING,		/**< String attribute. */
} kdump_attr_type_t;

/**  Dump file attribute value.
 */
typedef union _kdump_attr_value {
	kdump_num_t number;	/**< Valid if type is @ref KDUMP_NUMBER. */
	kdump_addr_t address;	/**< Valid if type is @ref KDUMP_ADDRESS.  */
	const char *string;	/**< Valid if type is @ref KDUMP_STRING. */
} kdump_attr_value_t;

/**  Dump file attribute: type + value.
 */
typedef struct _kdump_attr {
	kdump_attr_type_t type;	/**< Attribute type. */
	kdump_attr_value_t val;	/**< Attribute value. */
} kdump_attr_t;

/**  Reference to an attribute.
 * This type is used to make a fixed-size reference to an attribute,
 * rather than its (variable-size) key path.
 *
 * This type points to an internal structure which may change layout
 * without affecting the ABI, so callers must not make any attempts
 * to interpret that data.
 */
typedef struct _kdump_attr_ref {
	void *_ptr;		/**< Reference (private field). */
} kdump_attr_ref_t;

/**  Attribute iterator.
 * Iterators are used to iterate over all children of a directory
 * attribute. This is a public structure, so callers can allocate
 * it e.g. on stack.
 *
 * Note that the attribute name is stored in the structure, but
 * the attribute value is not. This allows to keep the same ABI
 * while implementing special attribute handling (e.g. calculate
 * the value on the fly).
 */
typedef struct _kdump_attr_iter {
	/** Attribute key.
	 * This is the attribute's name relative to parent (no dots),
	 * or @c NULL if end of iteration has been reached.
	 */
	const char *key;

	/** Iterator position.
	 * This field must not be modified by callers, but it can
	 * be used as an argument to the reference-handling functions.
	 */
	kdump_attr_ref_t pos;
} kdump_attr_iter_t;

/**  Set a dump file attribute.
 * @param ctx  Dump file object.
 * @param key  Attribute key.
 * @param valp New attribute value.
 * @returns    Error status.
 */
kdump_status kdump_set_attr(kdump_ctx_t *ctx, const char *key,
			    const kdump_attr_t *valp);

/**  Set a numeric attribute.
 * @param ctx  Dump file object.
 * @param key  Attribute key.
 * @param num  New attribute value.
 * @returns    Error status.
 */
static inline kdump_status
kdump_set_number_attr(kdump_ctx_t *ctx, const char *key, kdump_num_t num)
{
	kdump_attr_t attr;
	attr.type = KDUMP_NUMBER;
	attr.val.number = num;
	return kdump_set_attr(ctx, key, &attr);
}

/**  Set an address attribute.
 * @param ctx   Dump file object.
 * @param key   Attribute key.
 * @param addr  New attribute value.
 * @returns     Error status.
 */
static inline kdump_status
kdump_set_address_attr(kdump_ctx_t *ctx, const char *key, kdump_addr_t addr)
{
	kdump_attr_t attr;
	attr.type = KDUMP_ADDRESS;
	attr.val.address = addr;
	return kdump_set_attr(ctx, key, &attr);
}

/**  Set a string attribute.
 * @param ctx  Dump file object.
 * @param key  Attribute key.
 * @param str  New attribute value.
 * @returns    Error status.
 */
static inline kdump_status
kdump_set_string_attr(kdump_ctx_t *ctx, const char *key, const char *str)
{
	kdump_attr_t attr;
	attr.type = KDUMP_STRING;
	attr.val.string = str;
	return kdump_set_attr(ctx, key, &attr);
}

/**  Get a dump file attribute.
 * @param ctx  Dump file object.
 * @param key  Attribute key.
 * @param valp Value (filled on successful return).
 * @returns    Error status.
 *
 * Note that the caller does not hold a reference to the attribute, so
 * it is not generally safe to use this function in a multi-threaded
 * program, or across another library call which modifies the attribute
 * (explicitly or implicitly).
 */
kdump_status kdump_get_attr(kdump_ctx_t *ctx, const char *key,
			    kdump_attr_t *valp);

/** Get a numeric attribute.
 *
 * @param ctx       Dump file object.
 * @param key       Attribute key.
 * @param num[out]  Filled with the attribute value on successful return.
 * @returns         Error status.
 */
static inline kdump_status
kdump_get_number_attr(kdump_ctx_t *ctx, const char *key, kdump_num_t *num)
{
	kdump_attr_t attr;
	kdump_status ret = kdump_get_attr(ctx, key, &attr);
	if (ret == KDUMP_OK) {
		if (attr.type == KDUMP_NUMBER)
			*num = attr.val.number;
		else
			ret = KDUMP_ERR_INVALID;
	}
	return ret;
}

/** Get an address attribute.
 *
 * @param ctx        Dump file object.
 * @param key        Attribute key.
 * @param addr[out]  Filled with the attribute value on successful return.
 * @returns          Error status.
 *
 * Note that the caller does not hold a reference to the string. See
 * the description of @ref kdump_get_attr for limitations.
 */
static inline kdump_status
kdump_get_address_attr(kdump_ctx_t *ctx, const char *key, kdump_addr_t *addr)
{
	kdump_attr_t attr;
	kdump_status ret = kdump_get_attr(ctx, key, &attr);
	if (ret == KDUMP_OK) {
		if (attr.type == KDUMP_ADDRESS)
			*addr = attr.val.address;
		else
			ret = KDUMP_ERR_INVALID;
	}
	return ret;
}

/** Get a string attribute.
 *
 * @param ctx       Dump file object.
 * @param key       Attribute key.
 * @param str[out]  Filled with the attribute value on successful return.
 * @returns         Error status.
 *
 * Note that the caller does not hold a reference to the string. See
 * the description of @ref kdump_get_attr for limitations.
 */
static inline kdump_status
kdump_get_string_attr(kdump_ctx_t *ctx, const char *key, const char **str)
{
	kdump_attr_t attr;
	kdump_status ret = kdump_get_attr(ctx, key, &attr);
	if (ret == KDUMP_OK) {
		if (attr.type == KDUMP_STRING)
			*str = attr.val.string;
		else
			ret = KDUMP_ERR_INVALID;
	}
	return ret;
}

/** Get a reference to an attribute
 * @param      ctx  Dump file object.
 * @param[in]  key  Attribute key.
 * @param[out] ref  Attribute reference (initialized on successful return).
 * @returns    Error status.
 *
 * A reference is a persistent pointer to the attribute, which stays
 * valid until the reference is dropped using @ref kdump_attr_unref,
 * or the whole dump file object is destroyed.
 */
kdump_status kdump_attr_ref(kdump_ctx_t *ctx, const char *key,
			    kdump_attr_ref_t *ref);

/** Get a reference to a subordinate attribute
 * @param      ctx     Dump file object.
 * @param[in]  base    Reference to base attribute.
 * @param[in]  subkey  Attribute key, relative to @p base.
 * @param[out] ref     Attribute reference (initialized on successful return).
 * @returns    Error status.
 */
kdump_status kdump_sub_attr_ref(kdump_ctx_t *ctx, const kdump_attr_ref_t *base,
				const char *subkey, kdump_attr_ref_t *ref);

/**  Drop a reference to an attribute.
 * @param ctx   Dump file object.
 * @param ref   Attribute reference.
 */
void kdump_attr_unref(kdump_ctx_t *ctx, kdump_attr_ref_t *ref);

/**  Get the type of an attribute by reference.
 * @param ref  Attribute reference.
 * @returns    Attribute type.
 */
kdump_attr_type_t kdump_attr_ref_type(kdump_attr_ref_t *ref);

/**  Check whether an attribute is set.
 * @param ref  Attrbute reference.
 * @returns    Non-zero if the attribute has a value, zero otherwise.
 */
int kdump_attr_ref_isset(kdump_attr_ref_t *ref);

/**  Get attribute data by reference.
 * @param      ctx   Dump file object.
 * @param[in]  ref   Attribute reference.
 * @param[out] valp  Attribute value (filled on successful return).
 *
 * This works just like @ref kdump_get_attr, except that the attribute
 * is denoted by a reference rather than by its key path.
 */
kdump_status kdump_attr_ref_get(kdump_ctx_t *ctx, const kdump_attr_ref_t *ref,
				kdump_attr_t *valp);

/**  Set attribute data by reference.
 * @param      ctx   Dump file object.
 * @param[in]  ref   Attribute reference.
 * @param[in]  valp  New attribute value.
 *
 * This works just like @ref kdump_set_attr, except that the attribute
 * is denoted by a reference rather than by its key path.
 */
kdump_status kdump_attr_ref_set(kdump_ctx_t *ctx, kdump_attr_ref_t *ref,
				const kdump_attr_t *valp);

/**  Get an attribute iterator.
 * @param      ctx   Dump file object.
 * @param[in]  path  Path to an attribute directory.
 * @param[out] iter  Attribute iterator.
 * @returns          Error status.
 *
 * On return, the iterator is set to the first child attribute. If the
 * attribute directory is empty, this function sets the @c key field
 * of @p iter to @c NULL and returns @ref KDUMP_OK.
 */
kdump_status kdump_attr_iter_start(kdump_ctx_t *ctx, const char *path,
				   kdump_attr_iter_t *iter);

/**  Get an attribute iterator by reference.
 * @param      ctx   Dump file object.
 * @param[in]  ref   Reference to an attribute directory.
 * @param[out] iter  Attribute iterator.
 * @returns          Error status.
 *
 * This works just like @ref kdump_attr_iter_start, but use an
 * attribute reference rather than its key path.
 */
kdump_status kdump_attr_ref_iter_start(kdump_ctx_t *ctx,
				       const kdump_attr_ref_t *ref,
				       kdump_attr_iter_t *iter);

/**  Advance an attribute iterator.
 * @param ctx   Dump file object.
 * @param iter  Attribute iterator.
 * @returns     Error status.
 *
 * If there are no more items in the iteration, this function sets
 * the @c key field of @p iter to @c NULL and returns @ref KDUMP_OK.
 * If you try to advance past end of iteration, this function returns
 * @ref KDUMP_ERR_INVALID.
 */
kdump_status kdump_attr_iter_next(kdump_ctx_t *ctx, kdump_attr_iter_t *iter);

/**  De-initialize an attribute iterator.
 * @param ctx   Dump file object.
 * @param iter  Attribute iterator.
 * @returns     Error status.
 *
 * This function must be called when an iterator is no longer needed.
 */
void kdump_attr_iter_end(kdump_ctx_t *ctx, kdump_attr_iter_t *iter);

/**********************************************************************
 * WELL-KNOWN ATTRIBUTES
 *
 * Many well-known attribute names can be referenced by a macro.
 * These macros should be used instead of the string to minimize the
 * risk of typos. If you mistype an identifier, the compiler will
 * issue an error. If you mistype a string, it will just not work...
 */

/** File descriptor attribute. */
#define KDUMP_ATTR_FILE_FD	"file.fd"

/** File format name attribute.
 * This attribute is intended to be presented to humans rather than
 * machine-parsed. In fact, some format handlers create the string
 * dynamically, e.g. LKCD files will include the version in the string.
 */
#define KDUMP_ATTR_FILE_FORMAT	"file.format"

/** Canonical architecture name attribute.
 * Unlike @ref KDUMP_ATTR_MACHINE, which may contain the name of a
 * particular platform (e.g. "i586" v. "i686") or may not even be
 * initialised, this attribute (if set) always equals one of the
 * @c KDUMP_ARCH_xxx strings defined below.
 */
#define KDUMP_ATTR_ARCH_NAME	"arch.name"

/** @name Canonical Architecture Names
 * @{
 * Recognized values of the @c arch.name attribute. These macros should
 * be used instead of the actual string to prevent typos. Note that it
 * is not an error to set the @c arch.name attribute to a value that is
 * not recognized by the library. However, some functionality may be
 * unavailable (e.g. virtual-to-physical address translation).
 */
#define KDUMP_ARCH_AARCH64	"aarch64" /**< AArch64 (ARM64) */
#define KDUMP_ARCH_ALPHA	"alpha"	  /**< DEC Alpha */
#define KDUMP_ARCH_ARM		"arm"	  /**< ARM, 32-bit */
#define KDUMP_ARCH_IA32		"ia32"	  /**< Intel i386, i586, i686 */
#define KDUMP_ARCH_IA64		"ia64"	  /**< Intel Itanium */
#define KDUMP_ARCH_MIPS		"mips"	  /**< MIPS, 32-bit */
#define KDUMP_ARCH_PPC		"ppc"	  /**< Power ISA, 32-bit */
#define KDUMP_ARCH_PPC64	"ppc64"	  /**< Power ISA, 64-bit */
#define KDUMP_ARCH_S390		"s390"	  /**< IBM z/Architecture, 31-bit */
#define KDUMP_ARCH_S390X	"s390x"	  /**< IBM z/Architecture, 64-bit */
#define KDUMP_ARCH_X86_64	"x86_64"  /**< AMD64, Intel 64 */
/* @} */

/** Byte order attribute.
 * @sa kdump_byte_order_t
 */
#define KDUMP_ATTR_BYTE_ORDER	"arch.byte_order"

/** Pointer size attribute. */
#define KDUMP_ATTR_PTR_SIZE	"arch.ptr_size"

/** Page size attribute. */
#define KDUMP_ATTR_PAGE_SIZE	"arch.page_size"

/** Page shift attribute. */
#define KDUMP_ATTR_PAGE_SHIFT	"arch.page_shift"

/** Number of CPUs attribute.
 * The number is not the number of CPUs in the system, but rather the
 * number of CPUs with known register values.
 */
#define KDUMP_ATTR_NUM_CPUS	"cpu.number"

/** OS type attribute.
 */
#define KDUMP_ATTR_OSTYPE	"addrxlat.ostype"

/** Xen dump type file attribute.
 * @sa kdump_xen_type_t
 */
#define KDUMP_ATTR_XEN_TYPE	"xen.type"

/** Xen translation type attribute.
 * @sa kdump_xen_xlat_t
 */
#define KDUMP_ATTR_XEN_XLAT	"xen.xlat"

/** Linux version code attribute.
 * A Linux kernel version code is a 32-bit integer that combines the
 * first three digits of the kernel version.
 * See @c KERNEL_VERSION in <linux/version.h>.
 */
#define KDUMP_ATTR_LINUX_VERSION_CODE	"linux.version_code"

/** Xen version code attribute.
 * A Xen version code is a 32-bit integer that combines the major and
 * minor version.
 * See @c XENVER_version in <xen/version.h>.
 */
#define KDUMP_ATTR_XEN_VERSION_CODE	"xen.version_code"

/**  Get VMCOREINFO raw data.
 * @param ctx  Dump file object.
 * @returns    VMCOREINFO data, or @c NULL.
 *
 * The output string is always NUL-terminated, but if there was a NUL
 * byte inside VMCOREINFO, there is no way to know the full length.
 */
const char *kdump_vmcoreinfo(kdump_ctx_t *ctx);

/**  Get a VMCOREINFO row.
 * @param ctx      Dump file object.
 * @param[in] key  Name of the VMCOREINFO variable.
 * @returns        Value of the key, or @c NULL if not found.
 */
const char *kdump_vmcoreinfo_row(kdump_ctx_t *ctx, const char *key);

/**  Get VMCOREINFO symbol value.
 * @param ctx            Dump file object.
 * @param[in] symname    Kernel symbol name.
 * @param[out] symvalue  Value of the symbol.
 * @returns              Error status.
 *
 * Get the content of SYMBOL(@c symname) row and parse it as a hexadecimal
 * value.
 */
kdump_status kdump_vmcoreinfo_symbol(kdump_ctx_t *ctx, const char *symname,
				     kdump_addr_t *symvalue);

/**  Return the string describing a given error status.
 * @param status  Error status
 * @returns       Description of the error.
 */
const char *kdump_strerror(kdump_status status);

#ifdef  __cplusplus
}
#endif

#endif	/* kdumpfile.h */
